package com.huimv.farm.piglet.task;

import com.huimv.exception.PlatformException;
import com.huimv.farm.domain.constant.PigStatusEnum;
import com.huimv.farm.piglet.domain.Litter;
import com.huimv.farm.piglet.domain.Sow;
import com.huimv.farm.piglet.domain.count.ProductionWarning;
import com.huimv.farm.piglet.feign.clients.PublicFeignClient;
import com.huimv.farm.piglet.repo.LitterRepository;
import com.huimv.farm.piglet.repo.ProductionWarningRepository;
import com.huimv.farm.piglet.repo.SowRepository;
import com.huimv.publics.domain.SystemConfigData;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.*;

/**
 * 配置和提醒
 */
@Component
@SuppressWarnings("unused")
@Slf4j
public class CullingTask {

    @Autowired
    SowRepository sowRepository;

    @Autowired
    ProductionWarningRepository productionWarningRepository;

    @Autowired
    PublicFeignClient publicFeignClient;

    @Autowired
    LitterRepository litterRepository;

    //淘汰母猪条件设置
    //日龄
    private static final String OVER_AGE_KEY = "a";
    //胎次
    private static final String PARITY_KEY = "b";
    //分娩头数
    private static final String DELIVER_NUM_CONTINUOUS_KEY = "c1";
    private static final String DELIVER_NUM_TOTAL_KEY = "c2";
    private static final String DELIVER_NUM_THRESHOLD_KEY = "c3";
    //分娩存活率
    private static final String DELIVER_SURVIVE_CONTINUOUS_KEY = "d1";
    private static final String DELIVER_SURVIVE_TOTAL_KEY = "d2";
    private static final String DELIVER_SURVIVE_THRESHOLD_KEY = "d3";
    //断奶存活率
    private static final String WEANING_SURVIVE_CONTINUOUS_KEY = "e1";
    private static final String WEANING_SURVIVE_TOTAL_KEY = "e2";
    private static final String WEANING_SURVIVE_THRESHOLD_KEY = "e3";

    //生产条件设置
    //预产期
    private static final String DUE_KEY = "f";
    //分娩
    private static final String DELIVER_KEY = "g";
    //断奶
    private static final String WEANING_KEY = "h";
    //温度
    private static final String TEMPERATURE_KEY = "i";

    @Scheduled(cron = "0 0 2 * * ?")
    private void culling(){
        Integer overAge = getConf(OVER_AGE_KEY);
        if (null == overAge){
            log.info("淘汰母猪条件设置，按照日龄淘汰参数未设置");
            return;
        }
        List<String> needCulling = sowRepository.overAge(overAge);
        for (String no : needCulling){
            ProductionWarning one = new ProductionWarning();
            one.setMessage("【通知】母猪" + no + "日龄已超过" + overAge + "天，可淘汰");
            one.setStatus(0);
            one.setWarningTime(new Date());
            productionWarningRepository.save(one);
        }
    }

    @Scheduled(cron = "0 5 2 * * ?")
    private void parity(){
        Integer parity = getConf(PARITY_KEY);
        if (null == parity){
            log.info("淘汰母猪条件设置，按照分娩胎次淘汰参数未设置");
            return;
        }
        List<Integer> allSowId = litterRepository.cullingParity(parity);
        for (Integer sowId : allSowId){
            Optional<Sow> sow = sowRepository.findById(sowId);
            if (sow.isPresent() && sow.get().getStatus() != PigStatusEnum.STATUS_ELIMINATION.getCode()){
                saveWarning("【通知】母猪" + sow.get().getEarTagNo() + "分娩胎次超过" + parity + "次，可淘汰");
            }
        }
    }

    @Scheduled(cron = "0 10 2 * * ?")
    private void deliverNum(){
        Integer continuous = getConf(DELIVER_NUM_CONTINUOUS_KEY);
        Integer total = getConf(DELIVER_NUM_TOTAL_KEY);
        Integer threshold = getConf(DELIVER_NUM_THRESHOLD_KEY);
        if (null == continuous || null == total || null == threshold){
            log.info("淘汰母猪条件设置，按照分娩数量淘汰参数未设置");
            return;
        }
        //根据每胎数量淘汰
        //正在哺乳的不提示，两个条件不重复提示
        List<Integer> feeding = litterRepository.findMotherIdByDeliverTimeIsNotNullAndWeaningTimeIsNull();
        List<Integer> totalSow = litterRepository.cullingDeliverNumTotal(threshold, total);

        for (Integer sowId : totalSow){
            Optional<Sow> sow = sowRepository.findById(sowId);
            if (sow.isPresent() && sow.get().getStatus() != PigStatusEnum.STATUS_ELIMINATION.getCode()){
                saveWarning("【通知】母猪" + sow.get().getEarTagNo() + "累计" + total + "胎以上分娩仔猪在" + threshold + "头以下，可淘汰");
            }
        }

        List<Integer> exclude = new ArrayList<>();
        exclude.addAll(feeding);
        exclude.addAll(totalSow);

        List<Integer> toSearch = litterRepository.notInWeaning(exclude);
        for (Integer id : toSearch){
            List<Litter> lastTwo = litterRepository.lastTwoRecord(id);
            if (lastTwo.size() == 2 && lastTwo.get(0).getMother().getStatus() != PigStatusEnum.STATUS_ELIMINATION.getCode() && lastTwo.get(0).getDeliverNumber() < 9 && lastTwo.get(1).getDeliverNumber() < 9){
                saveWarning("【通知】母猪" + lastTwo.get(0).getMother().getEarTagNo() + "连续" + continuous + "胎分娩仔猪在" + threshold + "头以下，可淘汰");
            }
        }
    }

    @Scheduled(cron = "0 15 2 * * ?")
    private void deliverSurvive(){
        Integer continuous = getConf(WEANING_SURVIVE_CONTINUOUS_KEY);
        Integer total = getConf(WEANING_SURVIVE_TOTAL_KEY);
        Integer threshold = getConf(WEANING_SURVIVE_THRESHOLD_KEY);
        if (null == continuous || null == total || null == threshold){
            log.info("淘汰母猪条件设置，按照断奶存活率淘汰参数未设置");
            return;
        }
        //根据每胎数量淘汰
        //正在哺乳的不提示，两个条件不重复提示
        List<Integer> feeding = litterRepository.findMotherIdByDeliverTimeIsNotNullAndWeaningTimeIsNull();
        List<Integer> totalSow = litterRepository.cullingDeliverSurviveTotal(threshold, total);

        for (Integer sowId : totalSow){
            Optional<Sow> sow = sowRepository.findById(sowId);
            if (sow.isPresent() && sow.get().getStatus() != PigStatusEnum.STATUS_ELIMINATION.getCode()){
                saveWarning("【通知】母猪" + sow.get().getEarTagNo() + "累计" + total + "胎以上断奶存活率低于" + threshold + "以下，可淘汰");
            }
        }

        List<Integer> exclude = new ArrayList<>();
        exclude.addAll(feeding);
        exclude.addAll(totalSow);

        List<Integer> toSearch = litterRepository.notInWeaning(exclude);
        for (Integer id : toSearch){
            List<Litter> lastTwo = litterRepository.lastTwoRecord(id);
            if (lastTwo.size() == 2 && lastTwo.get(0).getMother().getStatus() != PigStatusEnum.STATUS_ELIMINATION.getCode()
                    && (lastTwo.get(0).getWeaningNumber() / (float)lastTwo.get(0).getWeaningNumber() * 100) < threshold
                    && (lastTwo.get(1).getWeaningNumber() / (float)lastTwo.get(1).getWeaningNumber() * 100) < threshold){
                saveWarning("【通知】母猪" + lastTwo.get(0).getMother().getEarTagNo() + "连续" + continuous + "胎断奶存活率低于" + threshold + "以下，可淘汰");
            }
        }
    }

    @Scheduled(cron = "0 20 2 * * ?")
    private void dayToDue(){
        Integer due = getConf(DUE_KEY);
        if (null == due){
            log.info("生产条件设置，预产期前提醒关注参数未设置");
            return;
        }

        List<Litter> toDue = litterRepository.periodToDue(due);
        for (Litter goingToDeliver : toDue){
            ProductionWarning one = new ProductionWarning();
            one.setMessage("【通知】母猪" + goingToDeliver.getMother().getEarTagNo() + "还有" + due + "天到达预产期");
            one.setStatus(0);
            one.setWarningTime(new Date());
            productionWarningRepository.save(one);
        }
    }

    @Scheduled(cron = "0 25 2 * * ?")
    private void dayAfterDeliver(){
        Integer deliver = getConf(DELIVER_KEY);
        if (null == deliver){
            log.info("生产条件设置，分娩后提醒断奶参数未设置");
            return;
        }

        List<Litter> afterDeliver = litterRepository.periodAfterDeliver(deliver);
        for (Litter goingToDeliver : afterDeliver){
            ProductionWarning one = new ProductionWarning();
            one.setMessage("【通知】母猪" + goingToDeliver.getMother().getEarTagNo() + "分娩已达" + deliver + "天，可断奶");
            one.setStatus(0);
            one.setWarningTime(new Date());
            productionWarningRepository.save(one);
        }
    }

    @Scheduled(cron = "0 30 2 * * ?")
    @Scheduled(cron = "*/5 * * * * ?")
    private void dayAfterWeaning(){
        Integer weaning = getConf(WEANING_KEY);
        if (null == weaning){
            log.info("生产条件设置，断奶后提醒转舍参数未设置");
            return;
        }

        List<Litter> afterWeaning = litterRepository.periodAfterWeaning(weaning);
        for (Litter oneWeaning : afterWeaning){
            ProductionWarning one = new ProductionWarning();
            one.setMessage("【通知】母猪" + oneWeaning.getMother().getEarTagNo() + "断奶已达" + weaning + "天，可转舍");
            one.setStatus(0);
            one.setWarningTime(new Date());
            productionWarningRepository.save(one);
        }
    }

    private Integer getConf(String key){
        SystemConfigData overAgeConf = publicFeignClient.get(null, 30009, null);
        if (null == overAgeConf){
            throw new PlatformException("can't find sow culling config", "找不到母猪淘汰条件设置");
        }

        Object str = overAgeConf.getConf().get("data");
        ArrayList array = (ArrayList)str;
        for (Object oneKey : array)
            if (key.equals(((HashMap)oneKey).get("key").toString())){
                return Integer.parseInt(String.valueOf(((HashMap)oneKey).get("val")));
            }
        return null;
    }

    private void saveWarning(String content){
        ProductionWarning one = new ProductionWarning();
        one.setMessage(content);
        one.setStatus(0);
        one.setWarningTime(new Date());
        productionWarningRepository.save(one);
    }
}
